/*      .
 *       ,      2.5 ,
 *   (        ),
 *    1602 */

#include <xc.h>
#include <stdint.h>

// CONFIG
//#pragma config FOSC = INTRCIO   // Oscillator Selection bits (INTOSC oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)
//#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
//#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
//#pragma config MCLRE = OFF      // RA3/MCLR pin function select (RA3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
//#pragma config BOREN = OFF      // Brown-out Detect Enable bit (BOD disabled)
//#pragma config CP = OFF         // Code Protection bit (Program Memory code protection is disabled)
//#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)

#define _XTAL_FREQ 1000000 //1Mhz FRC internal osc
#define __delay_us(x) _delay((unsigned long)((x)*(_XTAL_FREQ/1000000.0)))
#define __delay_ms(x) _delay((unsigned long)((x)*(_XTAL_FREQ/1000.0)))

#define RS    PORTCbits.RC4
#define E     PORTCbits.RC5
#define SIG   PORTAbits.RA0
#define AUTO  PORTAbits.RA1
#define F_KEY PORTAbits.RA4
#define S_KEY PORTAbits.RA5

// ,   "" 
    const uint8_t FLibr[4][8]={{0b11110,
                                0b00001,
                                0b00001,
                                0b00110,   //  
                                0b00001,
                                0b00001,
                                0b11110,
                                0b00000},
                           
                               {0b00111,
                                0b01001,
                                0b10001,
                                0b10001,    //  
                                0b10001,
                                0b10001,
                                0b10001,
                                0b00000},
                            
                               {0b10001,
                                0b10101,
                                0b10101,
                                0b01110,    //  
                                0b10101,
                                0b10101,
                                0b10001,
                                0b00000},
                            
                               {0b01110,
                                0b11111,
                                0b11111,
                                0b11111,    // 
                                0b11111,
                                0b11111,
                                0b01110,
                                0b00000}};

     
uint8_t *FLPoint; //    FLibr

//      ,    
const uint8_t Code_Lenght[5][6]={{0,0,0,0,0,0},         // 
                                 {35,12,24,12,24,79},   //
                                 {35,12,60,79},         //Ƹ
                                 {30,63,30,63},         //
                                 {255,255,255,255,255,255}};  //

//      .      Auto
//  ( ) -  (1-, 2-, 3-)
//  -  Auto
static uint8_t  Signal_Sec[3][4] = {{255,40,70,80}, {150,35,60,75,}, {30,30,30,20}}; 
     
void data_enter(uint8_t data)  
{
    E = 1;
    PORTC = PORTC & 0b110000 | data;
    __delay_us(30);
    E = 0;
    PORTC &= 0b110000;
}

//        4-  
void divide_n_enter(uint8_t data)
{
    data_enter(data>>4);
    data_enter(data&0b1111);
}
//        
void lcd_write_str(char*str) 
{
    do{
      divide_n_enter(*str);
    } while(*++str);
}

void init()
{
    ANSEL = 1;                  //   AN0(PORTA,0)  
    TRISA = 0b1111;             // RA0 -  , RA1 -     ,
                                // RA2 -       ,
    TRISC = 0;                  // RC4 - RS, RC5 - E;
    ADCON1 = 0b1010000;         //       1/16  Fosc
    ANSEL = 1;                  // RA0 -  
    OPTION_REG = 0b010;         // . . , TMR0  Fosc,  1/8
    INTCONbits.T0IE = 1;        //     TMR0
    T1CONbits.TMR1ON = 1;       //  TMR1
// ********** ***********
    RS = 0;
//    __delay_ms(50);
    data_enter(0b11);         //     8- 
    __delay_ms(20);
    data_enter(0b10);         //     4- 
    __delay_ms(20);
    divide_n_enter(0b1000);   //  
    __delay_ms(20);
    divide_n_enter(0b1);      //  
    __delay_ms(20);
    divide_n_enter(0b110);    //   
    __delay_ms(20);
    divide_n_enter(0b1100);   //  
    __delay_ms(20);
    divide_n_enter(0b101000); //  57, -  - 2
    __delay_ms(20);
    
//  ""    CGRAM
    divide_n_enter(0b1001000);      //    CGRAM    .    ( 8)      )
    RS = 1;
    for(uint8_t a; a<4; a++)
        for(uint8_t b=0; b<8; b++)                //   .
            divide_n_enter(FLibr[a][b]);          //     CGRAM   
}

uint8_t  Mode; // 0-, 1-, 2-Ƹ, 3-, 4-
uint8_t  Count_50Hz; //  Fosc=4,  TMR=1/8,   2 .  Count_50Hz=5 F_KEY    
uint8_t  Sec; //  -      
uint8_t  Auto_Mode; //   Auto.  3   .
uint8_t  Reset_to_White;  //    50 (1),    Auto        )
uint8_t  Code_Count;      // - 2      S_KEY
uint8_t Edge_Count;  //   
char ZEL[4] = {1,69,2,0};

void main ()
{
    init();
    TMR0=T0IF=0;
    ei();
    while(1)
    {
        if(Mode)
            if(Code_Count >= Code_Lenght[Mode][Edge_Count])
            {
                 S_KEY=~S_KEY;
                 Edge_Count++;
            }
        if(Code_Count>79) 
        {
            Edge_Count=0;
            S_KEY=1;
        }
        if(!SIG) 
            __delay_ms(300);
        if(!SIG) Mode++;
        Mode = Mode>4 ? 0 : Mode;
    
        if(!Mode && !AUTO)     
            __delay_ms(300);
        if(!Mode && !AUTO) 
        {
            Auto_Mode++;
            Reset_to_White++;
        }
        Auto_Mode = Auto_Mode>=3 ? 0 : Auto_Mode;
    
        
    
        RS=0;
        divide_n_enter(0b10000000); //       
        RS=1;
//    divide_n_enter(0x1); //    DDRAM ( CGRAM)   
    lcd_write_str(ZEL);
    }
}

void interrupt isr(void)
{
    Count_50Hz++;
    if(Count_50Hz>=5)
    {
        Code_Count++;
        Count_50Hz=0;
        F_KEY=~F_KEY;
    }
    T0IF=0;
}
